import pyjd
from pyjamas import DOM
from pyjamas import logging
logger = logging.getAppendLogger()

from pyjamas.ui.RootPanel import RootPanel, RootPanelCls, manageRootPanel
from pyjamas.ui.Button import Button
from pyjamas.ui.HTML import HTML
from pyjamas.ui.Label import Label
from pyjamas.ui.FocusPanel import FocusPanel
from pyjamas.Canvas.GWTCanvas import GWTCanvas
from pyjamas.Canvas.ImageLoader import loadImages, imageLoaders
from pyjamas.Canvas import Color

from pyjamas.Timer import Timer
from pyjamas import Window
from pyjamas.ui import Event
from pyjamas.ui import KeyboardListener
from pyjamas.ui.KeyboardListener import KeyboardHandler
from pyjamas.ui.ClickListener import ClickHandler
from pyjamas.ui.MouseListener import MouseHandler

from pyjamas.ui.Image import Image

from pyjamas.HTTPRequest import HTTPRequest

import math
import pygwt
import random

import sys
import game, player, tilemap, map_data

game.logger = logger
tilemap.logger = logger
FPS = 25.0

class Loader():
    def __init__(self):
        pass

    def onCompletion(self, doc):
        logger.debug("onCompletion")

    def onError(self, text, code):
        logger.debug("onError")

    def onTimeout(self, text):
        logger.debug("onTimeout")

class GameCanvas(GWTCanvas):
    def __init__(self, w, h):
        GWTCanvas.__init__(self, w, h)

        self.width = w
        self.height = h
        self.game = game.Game()
        self.game.canvas = self
        self.resized = False

        self.image_names = [x[0] for x in map_data.images]
        image_urls = ["./images/" + x + ".png" for x in self.image_names]
        loadImages(image_urls, self)
        self.run = False

        self.sinkEvents(Event.KEYEVENTS | Event.MOUSEEVENTS)

        t = Timer(0, self)
        t.scheduleRepeating(1000.0 / FPS)

    def onImagesLoaded(self, images):
        self.game.images = {}
        for i in range(len(images)):
            #logger.debug("%d: %s" % (i, self.image_names[i]))
            images[i].w = map_data.images[i][1]
            images[i].h = map_data.images[i][2]
            self.game.images[self.image_names[i]] = images[i]

        self.game.tilemap = tilemap.Tilemap()
        self.game.tilemap.read_data(map_data.map_data, self.game.images)

        self.game.init()

        self.run = True

    def addTo(self, panel):
        panel.add(self)
        self.top = DOM.getAbsoluteTop(self.getElement())
        self.left = DOM.getAbsoluteLeft(self.getElement())

    def onTimer(self, t = None):
        if not self.resized:
            self.resize(self.width, self.height)
            self.resized = True

        if not self.run:
            self.setFillStyle(Color.Color('#000'))
            self.fillRect(0, 0, self.width, self.height)

            i = imageLoaders[0].loadedImages
            n = imageLoaders[0].totalImages
            self.setFillStyle(Color.Color('#320'))
            self.fillRect(0, 0, self.width, 45)

            self.setFillStyle(Color.Color('#ff0'))
            self.fillRect(1, 1, (self.width - 2) * i / n, 45 - 2)
            return

        self.game.tick()

        self.draw()

    def set_key(self, k, on):
        DOM.eventPreventDefault(DOM.eventGetCurrentEvent())
        if k == KeyboardListener.KEY_UP:
            self.game.key_up = on
        elif k == KeyboardListener.KEY_DOWN:
            self.game.key_down = on
        elif k == KeyboardListener.KEY_LEFT:
            self.game.key_left = on
        elif k == KeyboardListener.KEY_RIGHT:
            self.game.key_right = on
        elif k == 32:
            self.game.key_fire = on

    def onMouseDown(self, sender, x, y):
        self.game.mx = self.game.scroll_x + x
        self.game.my = self.game.scroll_y + y
        self.game.click = True

    def onKeyPress(self, sender, k, modifiers = None):
        pass

    def onKeyDown(self, sender, k, modifiers = None):
        self.set_key(k, True)

    def onKeyUp(self, sender, k, modifiers = None):
        self.set_key(k, False)

    def draw(self):

        self.setFillStyle(Color.Color('#000'))
        self.fillRect(0, 0, self.width, self.height)

        self.game.draw(self)
